@@ -4,13 +4,15 @@
 #define TENATIVE 0 /* Code that is tenatively correct but hasn't yet been officially accepted */
 
 #define E820MAP	0x2d0		/* our map */
-#define E820MAX	128		/* number of entries in E820MAP */
 #define E820NR	0x1e8		/* # entries in E820MAP */
 
+#ifndef E820MAX
+#define E820MAX	128		/* number of entries in E820MAP */
+#endif
+
 #ifndef ASSEMBLY
 
-#define PACKED __attribute__((packed))
-
+#ifndef E820_RAM
 struct e820entry {
 	uint64_t addr;	/* start of memory segment */
 	uint64_t size;	/* size of memory segment */
@@ -19,7 +21,10 @@ struct e820entry {
 #define E820_RESERVED	2
 #define E820_ACPI	3 /* usable as RAM once ACPI tables have been read */
 #define E820_NVS	4
-} PACKED;
+#define E820_PMEM       7
+#define E820_PRAM       12
+} __attribute__((packed));
+#endif
 
 /* FIXME expand on drive_info_)struct... */
 struct drive_info_struct {
@@ -27,7 +32,7 @@ struct drive_info_struct {
 };
 struct sys_desc_table {
 	uint16_t length;
-	uint8_t  table[318];
+	uint8_t  table[30];
 };
 
 struct apm_bios_info {
@@ -40,7 +45,6 @@ struct apm_bios_info {
 	uint16_t cseg_len;	/* 0x4e */
 	uint16_t cseg_16_len;	/* 0x50 */
 	uint16_t dseg_len;	/* 0x52 */
-	uint8_t  reserved[44];	/* 0x54 */
 };
 
 /*
@@ -102,11 +106,22 @@ struct x86_linux_param_header {
 	uint16_t vesapm_seg;			/* 0x2e */
 	uint16_t vesapm_off;			/* 0x30 */
 	uint16_t pages;				/* 0x32 */
-	uint8_t  reserved4[12];			/* 0x34 -- 0x3f reserved for future expansion */
+	uint16_t vesa_attributes;		/* 0x34 */
+	uint32_t capabilities;			/* 0x36 */
+	uint32_t ext_lfb_base;			/* 0x3a */
+	uint8_t  reserved4[2];			/* 0x3e -- 0x3f reserved for future expansion */
 
 	struct apm_bios_info apm_bios_info;	/* 0x40 */
+	uint8_t  reserved4_1[28];		/* 0x54 */
+	uint64_t acpi_rsdp_addr;		/* 0x70 */
+	uint8_t  reserved4_2[8];		/* 0x78 */
 	struct drive_info_struct drive_info;	/* 0x80 */
 	struct sys_desc_table sys_desc_table;	/* 0xa0 */
+	uint32_t ext_ramdisk_image;		/* 0xc0 */
+	uint32_t ext_ramdisk_size;		/* 0xc4 */
+	uint32_t ext_cmd_line_ptr;		/* 0xc8 */
+	uint8_t reserved4_3[0x1c0 - 0xcc];	/* 0xe4 */
+	uint8_t efi_info[32];			/* 0x1c0 */
 	uint32_t alt_mem_k;			/* 0x1e0 */
 	uint8_t  reserved5[4];			/* 0x1e4 */
 	uint8_t  e820_map_nr;			/* 0x1e8 */
@@ -137,6 +152,7 @@ struct x86_linux_param_header {
 #define LOADER_TYPE_BOOTSECT_LOADER 2
 #define LOADER_TYPE_SYSLINUX        3
 #define LOADER_TYPE_ETHERBOOT       4
+#define LOADER_TYPE_KEXEC           0x0D
 #define LOADER_TYPE_UNKNOWN         0xFF
 	uint8_t  loader_flags;			/* 0x211 */
 	uint8_t  reserved12[2];			/* 0x212 */
@@ -169,11 +185,18 @@ struct x86_linux_param_header {
 	/* 2.04+ */
 	uint32_t kernel_alignment;		/* 0x230 */
 	uint8_t  relocatable_kernel;		/* 0x234 */
-	uint8_t  reserved15[3];			/* 0x235 */
+	uint8_t  min_alignment;			/* 0x235 */
+	uint16_t xloadflags;			/* 0x236 */
 	uint32_t cmdline_size;			/* 0x238 */
 	uint32_t hardware_subarch;		/* 0x23C */
 	uint64_t hardware_subarch_data;		/* 0x240 */
-	uint8_t  reserved16[0x290 - 0x248];	/* 0x248 */
+	uint32_t payload_offset;		/* 0x248 */
+	uint32_t payload_length;		/* 0x24C */
+	uint64_t setup_data;			/* 0x250 */
+	uint64_t pref_address;			/* 0x258 */
+	uint32_t init_size;			/* 0x260 */
+	uint32_t handover_offset;		/* 0x264 */
+	uint8_t  reserved16[0x290 - 0x268];	/* 0x268 */
 	uint32_t edd_mbr_sig_buffer[EDD_MBR_SIG_MAX];	/* 0x290 */
 #endif
 	struct 	e820entry e820_map[E820MAX];	/* 0x2d0 */
@@ -181,7 +204,7 @@ struct x86_linux_param_header {
 	struct 	edd_info eddbuf[EDDMAXNR];	/* 0xd00 */
 						/* 0xeec */
 #define COMMAND_LINE_SIZE 2048
-};
+} __attribute__((packed));
 
 struct x86_linux_faked_param_header {
 	struct x86_linux_param_header hdr;	/* 0x00 */
@@ -190,17 +213,20 @@ struct x86_linux_faked_param_header {
 };
 
 struct x86_linux_header {
-	uint8_t  reserved1[0x1f1];		/* 0x000 */
+	uint8_t  reserved1[0xc0];		/* 0x000 */
+	uint32_t ext_ramdisk_image;		/* 0x0c0 */
+	uint32_t ext_ramdisk_size;		/* 0x0c4 */
+	uint32_t ext_cmd_line_ptr;		/* 0x0c8 */
+	uint8_t  reserved1_1[0x1f1-0xcc];	/* 0x0cc */
 	uint8_t  setup_sects;			/* 0x1f1 */
 	uint16_t root_flags;			/* 0x1f2 */
-	uint16_t syssize;			/* 0x1f4 */
-	uint16_t swapdev;			/* 0x1f6 */
-	uint16_t ramdisk_flags;			/* 0x1f6 */
+	uint32_t syssize;			/* 0x1f4 */
+	uint16_t ram_size;			/* 0x1f8 */
 	uint16_t vid_mode;			/* 0x1fa */
 	uint16_t root_dev;			/* 0x1fc */
 	uint16_t boot_sector_magic;		/* 0x1fe */
 	/* 2.00+ */
-	uint8_t  reserved3[2];			/* 0x200 */
+	uint16_t jump;				/* 0x200 */
 	uint8_t  header_magic[4];		/* 0x202 */
 	uint16_t protocol_version;		/* 0x206 */
 	uint32_t realmode_swtch;		/* 0x208 */
@@ -212,38 +238,30 @@ struct x86_linux_header {
 	uint32_t code32_start;			/* 0x214 */
 	uint32_t ramdisk_image;			/* 0x218 */
 	uint32_t ramdisk_size;			/* 0x21c */
-	uint8_t  reserved4[4];			/* 0x220 */
+	uint32_t bootsect_kludge;		/* 0x220 */
 	/* 2.01+ */
 	uint16_t heap_end_ptr;			/* 0x224 */
-	uint8_t  reserved5[2];			/* 0x226 */
+	uint8_t  ext_loader_ver;		/* 0x226 */
+	uint8_t  ext_loader_type;		/* 0x227 */
 	/* 2.02+ */
 	uint32_t cmd_line_ptr;			/* 0x228 */
 	/* 2.03+ */
 	uint32_t initrd_addr_max;		/* 0x22c */
-#if TENATIVE
-	/* 2.04+ */
-	uint16_t entry32_off;			/* 0x230 */
-	uint16_t internal_cmdline_off;		/* 0x232 */
-	uint32_t low_base;			/* 0x234 */
-	uint32_t low_memsz;			/* 0x238 */
-	uint32_t low_filesz;			/* 0x23c */
-	uint32_t real_base;			/* 0x240 */
-	uint32_t real_memsz;			/* 0x244 */
-	uint32_t real_filesz;			/* 0x248 */
-	uint32_t high_base;			/* 0x24C */
-	uint32_t high_memsz;			/* 0x250 */
-	uint32_t high_filesz;			/* 0x254 */
-	uint32_t tail[32*1024 - 0x258];		/* 0x258 */
-#else
+
 	uint32_t kernel_alignment;		/* 0x230 */
 	uint8_t  relocatable_kernel;		/* 0x234 */
-	uint8_t  reserved6[3];			/* 0x235 */
-	uint32_t cmdline_size;                  /* 0x238 */
-	uint32_t hardware_subarch;              /* 0x23C */
-	uint64_t hardware_subarch_data;         /* 0x240 */
-	uint8_t  tail[32*1024 - 0x248];		/* 0x248 */
-#endif
-} PACKED;
+	uint8_t  min_alignment;			/* 0x235 */
+	uint16_t xloadflags;			/* 0x236 */
+	uint32_t cmdline_size;			/* 0x238 */
+	uint32_t hardware_subarch;		/* 0x23C */
+	uint64_t hardware_subarch_data;		/* 0x240 */
+	uint32_t payload_offset;		/* 0x248 */
+	uint32_t payload_size;			/* 0x24C */
+	uint64_t setup_data;			/* 0x250 */
+	uint64_t pref_address;			/* 0x258 */
+	uint32_t init_size;			/* 0x260 */
+	uint32_t handover_offset;		/* 0x264 */
+} __attribute__((packed));
 
 #endif /* ASSEMBLY */
 
