@@ -3,6 +3,7 @@
 
 #include <sys/types.h>
 
+#define SYSFS_FDT "/sys/firmware/fdt"
 #define BOOT_BLOCK_VERSION 17
 #define BOOT_BLOCK_LAST_COMP_VERSION 16
 
@@ -16,5 +17,6 @@ void zImage_arm_usage(void);
 int uImage_arm_probe(const char *buf, off_t len);
 int uImage_arm_load(int argc, char **argv, const char *buf, off_t len,
 		        struct kexec_info *info);
+extern int have_sysfs_fdt(void);
 
 #endif /* KEXEC_ARM_H */
